@@ -19,7 +19,7 @@
 */
 
 #include "module.h"
-#include "levels.h"
+#include <irssi/src/core/levels.h>
 
 /* the order of these levels must match the bits in levels.h */
 static const char *levels[] = {
@@ -61,6 +61,9 @@ int level_get(const char *level)
 	if (g_ascii_strcasecmp(level, "NO_ACT") == 0)
 		return MSGLEVEL_NO_ACT;
 
+	if (g_ascii_strcasecmp(level, "NOHILIGHT") == 0)
+		return MSGLEVEL_NOHILIGHT;
+
 	if (g_ascii_strcasecmp(level, "HIDDEN") == 0)
 		return MSGLEVEL_HIDDEN;
 
@@ -154,6 +157,9 @@ char *bits2level(int bits)
 		}
 	}
 
+	if (bits & MSGLEVEL_NOHILIGHT)
+		g_string_append(str, "NOHILIGHT ");
+
 	if (bits & MSGLEVEL_HIDDEN)
 		g_string_append(str, "HIDDEN ");
 
