@@ -19,16 +19,16 @@
 */
 
 #include "module.h"
-#include "commands.h"
-#include "network.h"
-#include "signals.h"
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/network.h>
+#include <irssi/src/core/signals.h>
 
-#include "chat-protocols.h"
-#include "servers.h"
-#include "servers-setup.h"
-#include "servers-reconnect.h"
+#include <irssi/src/core/chat-protocols.h>
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/servers-setup.h>
+#include <irssi/src/core/servers-reconnect.h>
 
-#include "settings.h"
+#include <irssi/src/core/settings.h>
 
 GSList *reconnects;
 static int last_reconnect_tag;
@@ -168,6 +168,7 @@ server_connect_copy_skeleton(SERVER_CONNECT_REC *src, int connect_info)
         server_connect_ref(dest);
 	dest->type = module_get_uniq_id("SERVER CONNECT", 0);
 	dest->reconnection = src->reconnection;
+	dest->last_failed_family = src->last_failed_family;
 	dest->proxy = g_strdup(src->proxy);
         dest->proxy_port = src->proxy_port;
 	dest->proxy_string = g_strdup(src->proxy_string);
@@ -238,8 +239,11 @@ static void sig_reconnect(SERVER_REC *server)
 	if (reconnect_time == -1 || !server_should_reconnect(server))
 		return;
 
-	conn = server_connect_copy_skeleton(server->connrec, FALSE);
-        g_return_if_fail(conn != NULL);
+	sserver = server_setup_find(server->connrec->address, server->connrec->port,
+	                            server->connrec->chatnet);
+
+	conn = server_connect_copy_skeleton(server->connrec, sserver == NULL);
+	g_return_if_fail(conn != NULL);
 
 	/* save the server status */
 	if (server->connected) {
@@ -248,10 +252,6 @@ static void sig_reconnect(SERVER_REC *server)
                 reconnect_save_status(conn, server);
 	}
 
-	sserver = server_setup_find(server->connrec->address,
-				    server->connrec->port,
-				    server->connrec->chatnet);
-
 	if (sserver != NULL) {
 		/* save the last connection time/status */
 		sserver->last_connect = server->connect_time == 0 ?
