@@ -19,19 +19,19 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "commands.h"
-#include "args.h"
-#include "network.h"
-#include "net-sendbuffer.h"
-#include "pidwait.h"
-#include "lib-config/iconfig.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/args.h>
+#include <irssi/src/core/network.h>
+#include <irssi/src/core/net-sendbuffer.h>
+#include <irssi/src/core/pidwait.h>
+#include <irssi/src/lib-config/iconfig.h>
 
-#include "chat-protocols.h"
-#include "servers.h"
-#include "servers-setup.h"
-#include "channels.h"
-#include "nicklist.h"
+#include <irssi/src/core/chat-protocols.h>
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/servers-setup.h>
+#include <irssi/src/core/channels.h>
+#include <irssi/src/core/nicklist.h>
 
 static char *session_file;
 char *irssi_binary = NULL;
@@ -254,23 +254,35 @@ static void session_restore_server(CONFIG_NODE *node)
 
 	proto = chat_protocol_find(chat_type);
 	if (proto == NULL || proto->not_initialized) {
-		if (handle < 0) close(handle);
+		if (handle >= 0)
+			close(handle);
 		return;
 	}
 
 	conn = server_create_conn(proto->id, address, port,
 				  chatnet, password, nick);
-	if (conn != NULL) {
-		conn->reconnection = TRUE;
-		conn->connect_handle = g_io_channel_new(handle);
+	if (conn == NULL)
+		return;
 
-		server = proto->server_init_connect(conn);
-		server->version = g_strdup(config_node_get_str(node, "version", NULL));
-		server->session_reconnect = TRUE;
-		signal_emit("session restore server", 2, server, node);
+	conn->use_tls = config_node_get_bool(node, "use_tls", FALSE);
+	conn->tls_cert = g_strdup(config_node_get_str(node, "tls_cert", NULL));
+	conn->tls_pkey = g_strdup(config_node_get_str(node, "tls_pkey", NULL));
+	conn->tls_verify = config_node_get_bool(node, "tls_verify", TRUE);
+	conn->tls_cafile = g_strdup(config_node_get_str(node, "tls_cafile", NULL));
+	conn->tls_capath = g_strdup(config_node_get_str(node, "tls_capath", NULL));
+	conn->tls_ciphers = g_strdup(config_node_get_str(node, "tls_ciphers", NULL));
+	conn->tls_pinned_cert = g_strdup(config_node_get_str(node, "tls_pinned_cert", NULL));
+	conn->tls_pinned_pubkey = g_strdup(config_node_get_str(node, "tls_pinned_pubkey", NULL));
 
-		proto->server_connect(server);
-	}
+	conn->reconnection = TRUE;
+	conn->connect_handle = i_io_channel_new(handle);
+
+	server = proto->server_init_connect(conn);
+	server->version = g_strdup(config_node_get_str(node, "version", NULL));
+	server->session_reconnect = TRUE;
+	signal_emit("session restore server", 2, server, node);
+
+	proto->server_connect(server);
 }
 
 static void sig_session_save(CONFIG_REC *config)
