@@ -19,14 +19,14 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "commands.h"
-#include "levels.h"
-#include "misc.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/misc.h>
 
-#include "lib-config/iconfig.h"
-#include "recode.h"
-#include "settings.h"
+#include <irssi/src/lib-config/iconfig.h>
+#include <irssi/src/core/recode.h>
+#include <irssi/src/core/settings.h>
 #include "default-config.h"
 
 #include <signal.h>
@@ -138,6 +138,23 @@ int settings_get_level(const char *key)
 	return str == NULL ? 0 : level2bits(str, NULL);
 }
 
+int settings_get_level_negative(const char *key)
+{
+	const char *str, *tmp, *all_levels;
+	int levels;
+
+	str = settings_get_str_type(key, SETTING_TYPE_LEVEL);
+	if (str == NULL)
+		return 0;
+
+	all_levels = bits2level(~0);
+	tmp = g_strdup_printf("%s %s", all_levels, str);
+	levels = level2bits(tmp, NULL) ^ level2bits(all_levels, NULL);
+	g_free((char *) tmp);
+	g_free((char *) all_levels);
+	return levels;
+}
+
 int settings_get_size(const char *key)
 {
 	const char *str;
@@ -229,6 +246,7 @@ static void settings_add(const char *module, const char *section,
 		if (rec->type != type) {
 			g_warning("Trying to add already existing "
 				  "setting '%s' with different type.", key);
+			g_strfreev(choices_vec);
 			return;
 		}
 		rec->refcount++;
@@ -556,7 +574,8 @@ static int backwards_compatibility(const char *module, CONFIG_NODE *node,
 			g_free(new_value);
 			config_changed = TRUE;
 			return new_key != NULL;
-		} else if (g_ascii_strcasecmp(node->key, "actlist_moves") == 0 &&
+		}
+		if (g_ascii_strcasecmp(node->key, "actlist_moves") == 0 &&
 			   node->value != NULL && g_ascii_strcasecmp(node->value, "yes") == 0) {
 			config_node_set_str(mainconfig, parent, "actlist_sort", "recent");
 			config_node_set_str(mainconfig, parent, node->key, NULL);
@@ -564,6 +583,12 @@ static int backwards_compatibility(const char *module, CONFIG_NODE *node,
 			return TRUE;
 		}
 	}
+	if (g_strcmp0(module, "core") == 0 &&
+			g_strcmp0(node->key, "resolve_reverse_lookup") == 0) {
+		config_node_set_str(mainconfig, parent, node->key, NULL);
+		config_changed = TRUE;
+		return TRUE;
+	}
 	return new_key != NULL;
 }
 
@@ -605,9 +630,8 @@ void settings_check_module(const char *module)
 		}
 	}
 	if (count > 0) {
-		if (gslist_find_icase_string(last_invalid_modules,
-					     module) == NULL) {
-                        /* mark this module having invalid settings */
+		if (i_slist_find_icase_string(last_invalid_modules, module) == NULL) {
+			/* mark this module having invalid settings */
 			last_invalid_modules =
 				g_slist_append(last_invalid_modules,
 					       g_strdup(module));
@@ -868,8 +892,7 @@ static int sig_autosave(void)
 
 void settings_init(void)
 {
-	settings = g_hash_table_new((GHashFunc) g_istr_hash,
-				    (GCompareFunc) g_istr_equal);
+	settings = g_hash_table_new((GHashFunc) i_istr_hash, (GCompareFunc) i_istr_equal);
 
 	last_errors = NULL;
         last_invalid_modules = NULL;
@@ -905,6 +928,7 @@ void settings_deinit(void)
 
 	g_hash_table_foreach(settings, (GHFunc) settings_hash_free, NULL);
 	g_hash_table_destroy(settings);
+	settings = NULL;
 
 	if (mainconfig != NULL) config_close(mainconfig);
 }
