@@ -19,25 +19,27 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "special-vars.h"
-#include "settings.h"
+#include <irssi/src/fe-common/core/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/special-vars.h>
+#include <irssi/src/core/settings.h>
 
-#include "levels.h"
-#include "servers.h"
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/servers.h>
 
-#include "fe-windows.h"
-#include "window-items.h"
-#include "formats.h"
-#include "themes.h"
-#include "recode.h"
-#include "utf8.h"
-#include "misc.h"
+#include <irssi/src/fe-common/core/fe-windows.h>
+#include <irssi/src/fe-common/core/window-items.h>
+#include <irssi/src/fe-common/core/formats.h>
+#include <irssi/src/fe-common/core/themes.h>
+#include <irssi/src/core/recode.h>
+#include <irssi/src/core/utf8.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/refstrings.h>
 
 static const char *format_backs = "04261537";
 static const char *format_fores = "kbgcrmyw";
 static const char *format_boldfores = "KBGCRMYW";
+static const char *ext_color_al = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 static int signal_gui_print_text;
 static int hide_text_style, hide_server_tags, hide_colors;
@@ -45,6 +47,8 @@ static int hide_text_style, hide_server_tags, hide_colors;
 static int timestamp_level;
 static int timestamp_timeout;
 
+static GHashTable *global_meta;
+
 int format_find_tag(const char *module, const char *tag)
 {
 	FORMAT_REC *formats;
@@ -121,10 +125,23 @@ void format_ext_color(GString *out, int bg, int color)
 					  : FORMAT_COLOR_EXT3);
 		g_string_append_c(out, FORMAT_COLOR_NOCHANGE + ((color-0x10)%0x50));
 	}
+
 	if (!bg && color < 0x10)
 		g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
 }
 
+static void format_ext_color_unexpand(GString *out, gboolean bg, int base, char color)
+{
+	unsigned char value = base + (unsigned char) color - FORMAT_COLOR_NOCHANGE - 0x10;
+
+	g_string_append_c(out, '%');
+	g_string_append_c(out, bg ? 'x' : 'X');
+	if (value > 214)
+		value += 10;
+	g_string_append_c(out, '1' + (value / 36));
+	g_string_append_c(out, ext_color_al[value % 36]);
+}
+
 #ifdef TERM_TRUECOLOR
 void unformat_24bit_color(char **ptr, int off, int *fgcolor, int *bgcolor, int *flags)
 {
@@ -152,6 +169,28 @@ void unformat_24bit_color(char **ptr, int off, int *fgcolor, int *bgcolor, int *
 		*flags |= GUI_PRINT_FLAG_COLOR_24_FG;
 	}
 }
+
+static void format_24bit_color_unexpand(GString *out, int off, const char **ptr)
+{
+	unsigned int color;
+	unsigned char rgbx[4];
+	unsigned int i;
+	for (i = 0; i < 4; ++i) {
+		if ((*ptr)[i + off] == '\0')
+			return;
+		rgbx[i] = (*ptr)[i + off];
+	}
+	rgbx[3] -= 0x20;
+	*ptr += 4;
+	g_string_append_c(out, '%');
+	for (i = 0; i < 3; ++i) {
+		if (rgbx[3] & (0x10 << i))
+			rgbx[i] -= 0x20;
+	}
+	color = rgbx[0] << 16 | rgbx[1] << 8 | rgbx[2];
+	g_string_append_c(out, rgbx[3] & 0x1 ? 'z' : 'Z');
+	g_string_append_printf(out, "%06X", color);
+}
 #endif
 
 void format_24bit_color(GString *out, int bg, unsigned int color)
@@ -308,6 +347,16 @@ int format_expand_styles(GString *out, const char **format, int *flags)
 
 		format_24bit_color(out, fmt == 'z', tmp2);
 		break;
+	case 'o':
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
+		g_string_append_c(out, (char) -1);
+		break;
+	case 'O':
+		g_string_append_c(out, 4);
+		g_string_append_c(out, (char) -1);
+		g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
+		break;
 	default:
 		/* check if it's a background color */
 		p = strchr(format_backs, fmt);
@@ -406,16 +455,31 @@ void format_read_arglist(va_list va, FORMAT_REC *format,
 		}
 	}
 }
-void format_create_dest(TEXT_DEST_REC *dest,
-			void *server, const char *target,
-			int level, WINDOW_REC *window)
+
+void format_dest_meta_stash(TEXT_DEST_REC *dest, const char *meta_key, const char *meta_value)
 {
-	format_create_dest_tag(dest, server, NULL, target, level, window);
+	g_hash_table_replace(dest->meta, i_refstr_intern(meta_key), g_strdup(meta_value));
 }
 
-void format_create_dest_tag(TEXT_DEST_REC *dest, void *server,
-			    const char *server_tag, const char *target,
-			    int level, WINDOW_REC *window)
+const char *format_dest_meta_stash_find(TEXT_DEST_REC *dest, const char *meta_key)
+{
+	return g_hash_table_lookup(dest->meta, meta_key);
+}
+
+void format_dest_meta_clear_all(TEXT_DEST_REC *dest)
+{
+	g_hash_table_remove_all(dest->meta);
+}
+
+static void clear_global_meta(WINDOW_REC *window, TEXT_DEST_REC *dest)
+{
+	if (dest != NULL && dest->meta == global_meta)
+		g_hash_table_remove_all(global_meta);
+}
+
+void format_create_dest_tag_meta(TEXT_DEST_REC *dest, void *server, const char *server_tag,
+                                 const char *target, int level, WINDOW_REC *window,
+                                 GHashTable *meta)
 {
 	memset(dest, 0, sizeof(TEXT_DEST_REC));
 
@@ -425,6 +489,20 @@ void format_create_dest_tag(TEXT_DEST_REC *dest, void *server,
 	dest->level = level;
 	dest->window = window != NULL ? window :
 		window_find_closest(server, target, level);
+	dest->meta = meta != NULL ? meta : global_meta;
+}
+
+void format_create_dest_tag(TEXT_DEST_REC *dest, void *server, const char *server_tag,
+                            const char *target, int level, WINDOW_REC *window)
+{
+	format_create_dest_tag_meta(dest, server, server_tag, target, level, window,
+	                            server != NULL ? SERVER(server)->current_incoming_meta : NULL);
+}
+
+void format_create_dest(TEXT_DEST_REC *dest, void *server, const char *target, int level,
+                        WINDOW_REC *window)
+{
+	format_create_dest_tag(dest, server, NULL, target, level, window);
 }
 
 /* Return length of text part in string (ie. without % codes) */
@@ -549,6 +627,123 @@ char *format_string_expand(const char *text, int *flags)
 	return ret;
 }
 
+inline static void format_flag_unexpand(GString *out, char flag)
+{
+	g_string_append_c(out, '%');
+	g_string_append_c(out, flag);
+}
+
+char *format_string_unexpand(const char *text, int flags)
+{
+	GString *out;
+
+	g_return_val_if_fail(text != NULL, NULL);
+
+	out = g_string_sized_new(strlen(text));
+	while (*text != '\0') {
+		switch (*text) {
+		case '%':
+			g_string_append(out, "%%");
+			break;
+		case 4:
+			text++;
+			if (*text == '\0')
+				break;
+			switch (*text) {
+			case FORMAT_COLOR_EXT1:
+				format_ext_color_unexpand(out, FALSE, 0x10, *++text);
+				break;
+			case FORMAT_COLOR_EXT1_BG:
+				format_ext_color_unexpand(out, TRUE, 0x10, *++text);
+				break;
+			case FORMAT_COLOR_EXT2:
+				format_ext_color_unexpand(out, FALSE, 0x60, *++text);
+				break;
+			case FORMAT_COLOR_EXT2_BG:
+				format_ext_color_unexpand(out, TRUE, 0x60, *++text);
+				break;
+			case FORMAT_COLOR_EXT3:
+				format_ext_color_unexpand(out, FALSE, 0xb0, *++text);
+				break;
+			case FORMAT_COLOR_EXT3_BG:
+				format_ext_color_unexpand(out, TRUE, 0xb0, *++text);
+				break;
+#ifdef TERM_TRUECOLOR
+			case FORMAT_COLOR_24:
+				format_24bit_color_unexpand(out, 1, &text);
+				break;
+#endif
+			case FORMAT_STYLE_BLINK:
+				format_flag_unexpand(out, 'F');
+				break;
+			case FORMAT_STYLE_UNDERLINE:
+				format_flag_unexpand(out, 'U');
+				break;
+			case FORMAT_STYLE_BOLD:
+				format_flag_unexpand(out, '9');
+				break;
+			case FORMAT_STYLE_REVERSE:
+				format_flag_unexpand(out, '8');
+				break;
+			case FORMAT_STYLE_INDENT:
+				format_flag_unexpand(out, '|');
+				break;
+			case FORMAT_STYLE_ITALIC:
+				format_flag_unexpand(out, 'I');
+				break;
+			case FORMAT_STYLE_DEFAULTS:
+				format_flag_unexpand(out, 'N');
+				break;
+			case FORMAT_STYLE_CLRTOEOL:
+				format_flag_unexpand(out, '>');
+				break;
+			case FORMAT_STYLE_MONOSPACE:
+				format_flag_unexpand(out, '#');
+				break;
+			default:
+				if (*text != FORMAT_COLOR_NOCHANGE) {
+					unsigned int value = (unsigned char) *text - '0';
+
+					g_string_append_c(out, '%');
+					if (value < 8) {
+						g_string_append_c(out, format_fores[value]);
+					} else if (value < 16) {
+						g_string_append_c(out, format_boldfores[value - 8]);
+					} else {
+						g_string_append_c(out, 'O');
+					}
+				}
+				text++;
+				if (*text == '\0')
+					break;
+
+				if (*text != FORMAT_COLOR_NOCHANGE) {
+					unsigned int value = (unsigned char) *text - '0';
+
+					g_string_append_c(out, '%');
+					if (value < 8) {
+						g_string_append_c(out, format_backs[value]);
+					} else if (value < 16) {
+						g_string_append(out, "x0");
+						g_string_append_c(out, ext_color_al[value]);
+					} else {
+						g_string_append_c(out, 'o');
+					}
+				}
+				break;
+			}
+			break;
+		default:
+			g_string_append_c(out, *text);
+			break;
+		}
+		if (*text != '\0')
+			text++;
+	}
+
+	return g_string_free(out, FALSE);
+}
+
 static char *format_get_text_args(TEXT_DEST_REC *dest,
 				  const char *text, char **arglist)
 {
@@ -650,6 +845,9 @@ char *format_get_text_theme_charargs(THEME_REC *theme, const char *module,
 	MODULE_THEME_REC *module_theme;
 	char *text;
 
+	if (module == NULL)
+		return NULL;
+
 	module_theme = g_hash_table_lookup(theme->modules, module);
 	if (module_theme == NULL)
 		return NULL;
@@ -850,14 +1048,13 @@ char *format_get_line_start(THEME_REC *theme, TEXT_DEST_REC *dest, time_t t)
 	return linestart;
 }
 
-void format_newline(WINDOW_REC *window)
+void format_newline(TEXT_DEST_REC *dest)
 {
-	g_return_if_fail(window != NULL);
+	g_return_if_fail(dest != NULL);
+	g_return_if_fail(dest->window != NULL);
 
-	signal_emit_id(signal_gui_print_text, 6, window,
-		       GINT_TO_POINTER(-1), GINT_TO_POINTER(-1),
-		       GINT_TO_POINTER(GUI_PRINT_FLAG_NEWLINE),
-		       "", NULL);
+	signal_emit_id(signal_gui_print_text, 6, dest->window, GINT_TO_POINTER(-1),
+	               GINT_TO_POINTER(-1), GINT_TO_POINTER(GUI_PRINT_FLAG_NEWLINE), "", dest);
 }
 
 #ifndef TERM_TRUECOLOR
@@ -1223,8 +1420,14 @@ char *strip_codes(const char *input)
 	return str;
 }
 
-/* send a fully parsed text string for GUI to print */
-void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
+/* parse text string into GUI_PRINT_FLAG_* separated pieces and emit them to handler
+   handler is a SIGNAL_FUNC with the following arguments:
+
+   WINDOW_REC *window, void *fgcolor_int, void *bgcolor_int,
+       void *flags_int, const char *textpiece, TEXT_DEST_REC *dest
+
+ */
+void format_send_as_gui_flags(TEXT_DEST_REC *dest, const char *text, SIGNAL_FUNC handler)
 {
 	THEME_REC *theme;
 	char *dup, *str, *ptr, type;
@@ -1239,8 +1442,8 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 
 	if (*str == '\0') {
 		/* empty line, write line info only */
-		signal_emit_id(signal_gui_print_text, 6, dest->window, GINT_TO_POINTER(fgcolor),
-		               GINT_TO_POINTER(bgcolor), GINT_TO_POINTER(flags), str, dest);
+		handler(dest->window, GINT_TO_POINTER(fgcolor), GINT_TO_POINTER(bgcolor),
+		        GINT_TO_POINTER(flags), str, dest);
 	}
 
 	while (*str != '\0') {
@@ -1260,16 +1463,14 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 
 		if (*str != '\0' || (flags & GUI_PRINT_FLAG_CLRTOEOL)) {
 			/* send the text to gui handler */
-			signal_emit_id(signal_gui_print_text, 6, dest->window,
-				       GINT_TO_POINTER(fgcolor),
-				       GINT_TO_POINTER(bgcolor),
-				       GINT_TO_POINTER(flags), str,
-				       dest);
+			handler(dest->window, GINT_TO_POINTER(fgcolor), GINT_TO_POINTER(bgcolor),
+			        GINT_TO_POINTER(flags), str, dest);
 			flags &= ~(GUI_PRINT_FLAG_INDENT|GUI_PRINT_FLAG_CLRTOEOL);
 		}
 
 		if (type == '\n') {
-			format_newline(dest->window);
+			handler(dest->window, GINT_TO_POINTER(-1), GINT_TO_POINTER(-1),
+			        GINT_TO_POINTER(GUI_PRINT_FLAG_NEWLINE), "", dest);
 			fgcolor = theme->default_color;
 			bgcolor = -1;
 			flags &= GUI_PRINT_FLAG_INDENT|GUI_PRINT_FLAG_MONOSPACE;
@@ -1415,6 +1616,104 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 	g_free(dup);
 }
 
+inline static void gui_print_text_emitter(WINDOW_REC *window, void *fgcolor_int, void *bgcolor_int,
+                                          void *flags_int, const char *textpiece,
+                                          TEXT_DEST_REC *dest)
+{
+	signal_emit_id(signal_gui_print_text, 6, window, fgcolor_int, bgcolor_int, flags_int,
+	               textpiece, dest);
+}
+
+/* send a fully parsed text string for GUI to print */
+void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
+{
+	format_send_as_gui_flags(dest, text, (SIGNAL_FUNC) gui_print_text_emitter);
+}
+
+void format_gui_flags(GString *out, int *last_fg, int *last_bg, int *last_flags, int fg, int bg,
+                      int flags)
+{
+	if (fg != *last_fg ||
+	    (flags & GUI_PRINT_FLAG_COLOR_24_FG) != (*last_flags & GUI_PRINT_FLAG_COLOR_24_FG)) {
+		*last_fg = fg;
+
+#ifdef TERM_TRUECOLOR
+		if (flags & GUI_PRINT_FLAG_COLOR_24_FG) {
+			*last_flags |= GUI_PRINT_FLAG_COLOR_24_FG;
+			format_24bit_color(out, 0, fg);
+		} else {
+			*last_flags &= ~GUI_PRINT_FLAG_COLOR_24_FG;
+#endif
+			if (fg < 0) {
+				g_string_append_c(out, 4);
+				g_string_append_c(out, (char) -1);
+				g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
+			} else {
+				format_ext_color(out, 0, fg);
+			}
+#ifdef TERM_TRUECOLOR
+		}
+#endif
+	}
+	if (bg != *last_bg ||
+	    (flags & GUI_PRINT_FLAG_COLOR_24_BG) != (*last_flags & GUI_PRINT_FLAG_COLOR_24_BG)) {
+		*last_bg = bg;
+#ifdef TERM_TRUECOLOR
+		if (flags & GUI_PRINT_FLAG_COLOR_24_BG) {
+			*last_flags |= GUI_PRINT_FLAG_COLOR_24_BG;
+			format_24bit_color(out, 1, bg);
+		} else {
+			*last_flags &= ~GUI_PRINT_FLAG_COLOR_24_BG;
+#endif
+			if (bg < 0) {
+				g_string_append_c(out, 4);
+				g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
+				g_string_append_c(out, (char) -1);
+			} else {
+				format_ext_color(out, 1, bg);
+			}
+#ifdef TERM_TRUECOLOR
+		}
+#endif
+	}
+
+	if ((flags & GUI_PRINT_FLAG_UNDERLINE) != (*last_flags & GUI_PRINT_FLAG_UNDERLINE)) {
+		*last_flags ^= GUI_PRINT_FLAG_UNDERLINE;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_UNDERLINE);
+	}
+	if ((flags & GUI_PRINT_FLAG_REVERSE) != (*last_flags & GUI_PRINT_FLAG_REVERSE)) {
+		*last_flags ^= GUI_PRINT_FLAG_REVERSE;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_REVERSE);
+	}
+	if ((flags & GUI_PRINT_FLAG_BLINK) != (*last_flags & GUI_PRINT_FLAG_BLINK)) {
+		*last_flags ^= GUI_PRINT_FLAG_BLINK;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_BLINK);
+	}
+	if ((flags & GUI_PRINT_FLAG_BOLD) != (*last_flags & GUI_PRINT_FLAG_BOLD)) {
+		*last_flags ^= GUI_PRINT_FLAG_BOLD;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_BOLD);
+	}
+	if ((flags & GUI_PRINT_FLAG_ITALIC) != (*last_flags & GUI_PRINT_FLAG_ITALIC)) {
+		*last_flags ^= GUI_PRINT_FLAG_ITALIC;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_ITALIC);
+	}
+	if ((flags & GUI_PRINT_FLAG_MONOSPACE) != (*last_flags & GUI_PRINT_FLAG_MONOSPACE)) {
+		*last_flags ^= GUI_PRINT_FLAG_MONOSPACE;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_MONOSPACE);
+	}
+	if (flags & GUI_PRINT_FLAG_INDENT) {
+		*last_flags ^= GUI_PRINT_FLAG_INDENT;
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_INDENT);
+	}
+}
+
 static void read_settings(void)
 {
 	timestamp_level = settings_get_bool("timestamps") ? MSGLEVEL_ALL : 0;
@@ -1430,12 +1729,18 @@ static void read_settings(void)
 void formats_init(void)
 {
 	signal_gui_print_text = signal_get_uniq_id("gui print text");
+	global_meta =
+	    g_hash_table_new_full(g_str_hash, (GEqualFunc) g_str_equal,
+	                          (GDestroyNotify) i_refstr_release, (GDestroyNotify) g_free);
 
 	read_settings();
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
+	signal_add_last("gui print text finished", (SIGNAL_FUNC) clear_global_meta);
 }
 
 void formats_deinit(void)
 {
+	g_hash_table_destroy(global_meta);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
+	signal_remove("gui print text finished", (SIGNAL_FUNC) clear_global_meta);
 }
