@@ -19,17 +19,17 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "commands.h"
-#include "levels.h"
-#include "misc.h"
-#include "lib-config/iconfig.h"
-#include "settings.h"
+#include <irssi/src/fe-common/core/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/lib-config/iconfig.h>
+#include <irssi/src/core/settings.h>
 
-#include "keyboard.h"
-#include "fe-windows.h"
-#include "printtext.h"
+#include <irssi/src/fe-common/core/keyboard.h>
+#include <irssi/src/fe-common/core/fe-windows.h>
+#include <irssi/src/fe-common/core/printtext.h>
 
 #define MAX_EXPAND_RECURSION 100
 
@@ -770,9 +770,8 @@ static void cmd_show_keys(const char *searchkey, int full)
 		for (key = rec->keys; key != NULL; key = key->next) {
 			KEY_REC *rec = key->data;
 
-			if ((len == 0 ||
-			     (full ? strncmp(rec->key, searchkey, len) == 0 :
-			             g_ascii_strncasecmp(rec->key, searchkey, len) == 0)) &&
+			if ((len == 0 || (full ? strncmp(rec->key, searchkey, len) == 0 :
+			                         strstr(rec->key, searchkey) != NULL)) &&
 			    (!full || rec->key[len] == '\0')) {
 				printformat(NULL, NULL, MSGLEVEL_CLIENTCRAP, TXT_BIND_LIST,
 					    rec->key, rec->info->id, rec->data == NULL ? "" : rec->data);
