@@ -19,9 +19,10 @@
 */
 
 #include "module.h"
-#include "formats.h"
+#include <irssi/src/fe-common/core/formats.h>
 
 FORMAT_REC fecommon_core_formats[] = {
+	/* clang-format off */
 	{ MODULE_NAME, "Core", 0 },
 
 	/* ---- */
@@ -87,24 +88,32 @@ FORMAT_REC fecommon_core_formats[] = {
 	{ "server_reconnect_removed", "Removed reconnection to server {server $0} port {hilight $1}", 3, { 0, 1, 0 } },
 	{ "server_reconnect_not_found", "Reconnection tag {server $0} not found", 1, { 0 } },
 	{ "setupserver_added", "Server {server $0} saved", 2, { 0, 1 } },
-	{ "setupserver_removed", "Server {server $0} removed", 2, { 0, 1 } },
-	{ "setupserver_not_found", "Server {server $0} not found", 2, { 0, 1 } },
+	{ "setupserver_removed", "Server {server $0} {hilight $1} removed", 2, { 0, 1 } },
+	{ "setupserver_not_found", "Server {server $0} {hilight $1} not found", 2, { 0, 1 } },
 	{ "your_nick", "Your nickname is {nick $0}", 1, { 0 } },
 
 	/* ---- */
 	{ NULL, "Channels", 0 },
 
-	{ "join", "{channick_hilight $0} {chanhost_hilight $1} has joined {channel $2}", 3, { 0, 0, 0 } },
+	{ "join", "{channick_hilight $0} {chanhost_hilight $1} has joined {channel $2}", 5, { 0, 0, 0, 0, 0 } },
+	{ "join_extended", "{channick_hilight $0} {chanhost_hilight $1} has joined {channel $2} {comment realname {reason $4}}", 5, { 0, 0, 0, 0, 0 } },
+	{ "join_extended_account", "{channick_hilight $0} {chanhost_hilight $1} has joined {channel $2} {reason account {hilight $3}} {comment realname {reason $4}}", 5, { 0, 0, 0, 0, 0 } },
+	{ "host_changed", "{channick_hilight $0} {chanhost_hilight $1} has changed host", 4, { 0, 0, 0, 0 } },
+	{ "logged_out", "{channick $0} {chanhost $1} has logged out of their account", 4, { 0, 0, 0, 0 } },
+	{ "logged_in", "{channick_hilight $0} {chanhost_hilight $1} has logged in to account {hilight $2}", 4, { 0, 0, 0, 0 } },
 	{ "part", "{channick $0} {chanhost $1} has left {channel $2} {reason $3}", 4, { 0, 0, 0, 0 } },
 	{ "kick", "{channick $0} was kicked from {channel $1} by {nick $2} {reason $3}", 5, { 0, 0, 0, 0, 0 } },
 	{ "quit", "{channick $0} {chanhost $1} has quit {reason $2}", 4, { 0, 0, 0, 0 } },
 	{ "quit_once", "{channel $3} {channick $0} {chanhost $1} has quit {reason $2}", 4, { 0, 0, 0, 0 } },
 	{ "invite", "{nick $0} invites you to {channel $1}", 3, { 0, 0, 0 } },
 	{ "not_invited", "You have not been invited to a channel!", 0 },
+	{ "invite_other", "{nick $0} has been invited to {channel $2} by {channick_hilight $1}", 4, { 0, 0, 0, 0 } },
 	{ "new_topic", "{nick $0} changed the topic of {channel $1} to: $2", 4, { 0, 0, 0, 0 } },
 	{ "topic_unset", "Topic unset by {nick $0} on {channel $1}", 4, { 0, 0, 0, 0 } },
 	{ "your_nick_changed", "You're now known as {nick $1}", 4, { 0, 0, 0, 0 } },
 	{ "nick_changed", "{channick $0} is now known as {channick_hilight $1}", 4, { 0, 0, 0, 0 } },
+	{ "notify_away_channel", "{channick $0} {chanhost $1} is now away: {reason $2}", 4, { 0, 0, 0, 0 } },
+	{ "notify_unaway_channel", "{channick_hilight $0} {chanhost $1} is no longer away", 4, { 0, 0, 0, 0 } },
 	{ "talking_in", "You are now talking in {channel $0}", 1, { 0 } },
 	{ "not_in_channels", "You are not on any channels", 0 },
 	{ "current_channel", "Current channel {channel $0}", 1, { 0 } },
@@ -279,7 +288,7 @@ FORMAT_REC fecommon_core_formats[] = {
 	{ "config_saved", "Saved configuration to file $0", 1, { 0 } },
 	{ "config_reloaded", "Reloaded configuration", 1, { 0 } },
 	{ "config_modified", "Configuration file was modified since irssi was last started - do you want to overwrite the possible changes?", 1, { 0 } },
-	{ "glib_error", "{error $0} $1", 2, { 0, 0 } },
+	{ "glib_error", "{error ($0) $1} $2", 3, { 0, 0, 0 } },
 	{ "overwrite_config", "Overwrite config (y/N)?", 0 },
 	{ "set_title", "[{hilight $0}]", 1, { 0 } },
 	{ "set_item", "$[-!32]0 %_$1", 2, { 0, 0 } },
@@ -308,4 +317,5 @@ FORMAT_REC fecommon_core_formats[] = {
 	{ "tls_protocol_version", "Protocol: {hilight $0} ({hilight $1} bit, {hilight $2})", 3, { 0, 1, 0 } },
 
 	{ NULL, NULL, 0 }
+	/* clang-format on */
 };
