@@ -19,29 +19,31 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "modules.h"
-#include "signals.h"
-#include "commands.h"
-#include "settings.h"
+#include <irssi/src/fe-common/core/module-formats.h>
+#include <irssi/src/core/modules.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/settings.h>
 
-#include "levels.h"
-#include "servers.h"
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/servers.h>
 
-#include "themes.h"
-#include "fe-windows.h"
-#include "printtext.h"
+#include <irssi/src/fe-common/core/themes.h>
+#include <irssi/src/fe-common/core/fe-windows.h>
+#include <irssi/src/fe-common/core/printtext.h>
 
-static int beep_msg_level, beep_when_away, beep_when_window_active;
+static int beep_msg_level, beep_msg_level_ignore, beep_when_away, beep_when_window_active;
 
 static int signal_gui_print_text_finished;
 static int signal_print_starting;
 static int signal_print_text;
 static int signal_print_format;
+static int signal_print_noformat;
+static int signal_window_hilight_check;
 
 static int sending_print_starting;
 
-static void print_line(TEXT_DEST_REC *dest, const char *text);
+static void print_line(TEXT_DEST_REC *dest, const char *text, int formatted);
 
 void printformat_module_dest_args(const char *module, TEXT_DEST_REC *dest,
 				  int formatnum, va_list va)
@@ -62,7 +64,6 @@ void printformat_module_dest_charargs(const char *module, TEXT_DEST_REC *dest,
 				      int formatnum, char **arglist)
 {
 	THEME_REC *theme;
-	char *str;
 
 	theme = window_get_theme(dest->window);
 
@@ -74,11 +75,6 @@ void printformat_module_dest_charargs(const char *module, TEXT_DEST_REC *dest,
 
 	signal_emit_id(signal_print_format, 5, theme, module,
 		       dest, GINT_TO_POINTER(formatnum), arglist);
-
-	str = format_get_text_theme_charargs(theme, module, dest,
-					     formatnum, arglist);
-	if (str != NULL && *str != '\0') print_line(dest, str);
-	g_free(str);
 }
 
 void printformat_module_dest(const char *module, TEXT_DEST_REC *dest,
@@ -163,29 +159,6 @@ void printformat_module_gui(const char *module, int formatnum, ...)
 	va_end(va);
 }
 
-static void print_line(TEXT_DEST_REC *dest, const char *text)
-{
-        THEME_REC *theme;
-	char *str, *tmp, *stripped;
-
-	g_return_if_fail(dest != NULL);
-	g_return_if_fail(text != NULL);
-
-        theme = window_get_theme(dest->window);
-	tmp = format_get_level_tag(theme, dest);
-	str = !theme->info_eol ? format_add_linestart(text, tmp) :
-		format_add_lineend(text, tmp);
-	g_free_not_null(tmp);
-
-	/* send both the formatted + stripped (for logging etc.) */
-	stripped = strip_codes(str);
-	signal_emit_id(signal_print_text, 3, dest, str, stripped);
-        g_free_and_null(dest->hilight_color);
-
-	g_free(str);
-        g_free(stripped);
-}
-
 /* append string to `out', expand newlines. */
 static void printtext_append_str(TEXT_DEST_REC *dest, GString *out,
 				 const char *str)
@@ -194,7 +167,7 @@ static void printtext_append_str(TEXT_DEST_REC *dest, GString *out,
 		if (*str != '\n')
 			g_string_append_c(out, *str);
 		else {
-			print_line(dest, out->str);
+			print_line(dest, out->str, 0);
 			g_string_truncate(out, 0);
 		}
 		str++;
@@ -313,7 +286,7 @@ static void printtext_dest_args(TEXT_DEST_REC *dest, const char *text, va_list v
 	}
 
 	str = printtext_get_args(dest, text, va);
-	print_line(dest, str);
+	print_line(dest, str, 0);
 	g_free(str);
 }
 
@@ -358,8 +331,8 @@ void printtext_string(void *server, const char *target, int level, const char *t
 	}
 
         str = printtext_expand_formats(text, &dest.flags);
-	print_line(&dest, str);
-        g_free(str);
+	print_line(&dest, str, 0);
+	g_free(str);
 }
 
 /* Like printtext_window(), but don't handle %s etc. */
@@ -380,8 +353,8 @@ void printtext_string_window(WINDOW_REC *window, int level, const char *text)
 	}
 
         str = printtext_expand_formats(text, &dest.flags);
-	print_line(&dest, str);
-        g_free(str);
+	print_line(&dest, str, 0);
+	g_free(str);
 }
 
 void printtext_window(WINDOW_REC *window, int level, const char *text, ...)
@@ -432,7 +405,18 @@ static void msg_beep_check(TEXT_DEST_REC *dest)
 	    (beep_when_away || (dest->server != NULL &&
 				!dest->server->usermode_away)) &&
 	    (beep_when_window_active || dest->window != active_win)) {
-                signal_emit("beep", 0);
+		if (beep_msg_level_ignore & MSGLEVEL_HIDDEN) {
+			int cb_ignore = 0;
+			if (signal_emit_id(signal_window_hilight_check, 4, dest, NULL, NULL,
+			                   &cb_ignore),
+			    cb_ignore) {
+				return;
+			}
+		} else if (beep_msg_level_ignore & dest->level) {
+			return;
+		}
+
+		signal_emit("beep", 0);
 	}
 }
 
@@ -447,7 +431,7 @@ static void sig_print_text(TEXT_DEST_REC *dest, const char *text)
 	if (dest->window == NULL) {
                 str = strip_codes(text);
 #ifndef SUPPRESS_PRINTF_FALLBACK
-		printf("NO WINDOWS: %s\n", str);
+		printf("## NO WINDOWS: %s\n", str);
 #endif
                 g_free(str);
                 return;
@@ -470,7 +454,52 @@ static void sig_print_text(TEXT_DEST_REC *dest, const char *text)
 	format_send_to_gui(dest, str);
 	g_free(str);
 
-	signal_emit_id(signal_gui_print_text_finished, 1, dest->window);
+	signal_emit_id(signal_gui_print_text_finished, 2, dest->window, dest);
+}
+
+static void sig_print_format(THEME_REC *theme, const char *module, TEXT_DEST_REC *dest,
+                             void *formatnump, char **arglist)
+{
+	int formatnum;
+	char *str;
+
+	formatnum = GPOINTER_TO_INT(formatnump);
+	str = format_get_text_theme_charargs(theme, module, dest, formatnum, arglist);
+	if (str != NULL && *str != '\0')
+		print_line(dest, str, 1);
+
+	g_free(str);
+}
+
+static void sig_print_noformat(TEXT_DEST_REC *dest, const char *text)
+{
+	THEME_REC *theme;
+	char *str, *tmp, *stripped;
+
+	theme = window_get_theme(dest->window);
+	tmp = format_get_level_tag(theme, dest);
+	str = !theme->info_eol ? format_add_linestart(text, tmp) : format_add_lineend(text, tmp);
+	g_free_not_null(tmp);
+
+	/* send both the formatted + stripped (for logging etc.) */
+	stripped = strip_codes(str);
+	signal_emit_id(signal_print_text, 3, dest, str, stripped);
+
+	g_free_and_null(dest->hilight_color);
+
+	g_free(str);
+	g_free(stripped);
+}
+
+static void print_line(TEXT_DEST_REC *dest, const char *text, int formatted)
+{
+	g_return_if_fail(dest != NULL);
+	g_return_if_fail(text != NULL);
+
+	if (!formatted)
+		signal_emit_id(signal_print_noformat, 2, dest, text);
+	else
+		sig_print_noformat(dest, text);
 }
 
 void printtext_multiline(void *server, const char *target, int level,
@@ -504,6 +533,7 @@ static void sig_gui_dialog(const char *type, const char *text)
 static void read_settings(void)
 {
 	beep_msg_level = settings_get_level("beep_msg_level");
+	beep_msg_level_ignore = settings_get_level_negative("beep_msg_level");
 	beep_when_away = settings_get_bool("beep_when_away");
         beep_when_window_active = settings_get_bool("beep_when_window_active");
 }
@@ -515,9 +545,13 @@ void printtext_init(void)
 	signal_print_starting = signal_get_uniq_id("print starting");
 	signal_print_text = signal_get_uniq_id("print text");
 	signal_print_format = signal_get_uniq_id("print format");
+	signal_print_noformat = signal_get_uniq_id("print noformat");
+	signal_window_hilight_check = signal_get_uniq_id("window hilight check");
 
 	read_settings();
 	signal_add("print text", (SIGNAL_FUNC) sig_print_text);
+	signal_add("print format", (SIGNAL_FUNC) sig_print_format);
+	signal_add("print noformat", (SIGNAL_FUNC) sig_print_noformat);
 	signal_add("gui dialog", (SIGNAL_FUNC) sig_gui_dialog);
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
 }
@@ -525,6 +559,8 @@ void printtext_init(void)
 void printtext_deinit(void)
 {
 	signal_remove("print text", (SIGNAL_FUNC) sig_print_text);
+	signal_remove("print format", (SIGNAL_FUNC) sig_print_format);
+	signal_remove("print noformat", (SIGNAL_FUNC) sig_print_noformat);
 	signal_remove("gui dialog", (SIGNAL_FUNC) sig_gui_dialog);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
 }
