@@ -19,22 +19,22 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "commands.h"
-#include "misc.h"
+#include <irssi/src/fe-common/irc/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/misc.h>
 
-#include "servers-setup.h"
+#include <irssi/src/core/servers-setup.h>
 
-#include "levels.h"
-#include "irc-chatnets.h"
-#include "irc-servers.h"
-#include "irc-channels.h"
-#include "servers-reconnect.h"
-#include "irc-servers-setup.h"
+#include <irssi/src/core/levels.h>
+#include <irssi/src/irc/core/irc-chatnets.h>
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/irc-channels.h>
+#include <irssi/src/core/servers-reconnect.h>
+#include <irssi/src/irc/core/irc-servers-setup.h>
 
-#include "fe-windows.h"
-#include "printtext.h"
+#include <irssi/src/fe-common/core/fe-windows.h>
+#include <irssi/src/fe-common/core/printtext.h>
 
 const char *get_visible_target(IRC_SERVER_REC *server, const char *target)
 {
@@ -51,12 +51,13 @@ const char *get_visible_target(IRC_SERVER_REC *server, const char *target)
 	return target;
 }
 
-/* SYNTAX: SERVER ADD|MODIFY [-4 | -6] [-ssl] [-ssl_cert <cert>] [-ssl_pkey <pkey>] [-ssl_pass <password>]
-                             [-ssl_verify] [-ssl_cafile <cafile>] [-ssl_capath <capath>]
-                             [-ssl_ciphers <list>]
+/* SYNTAX: SERVER ADD|MODIFY [-4 | -6] [-cap | -nocap] [-tls_cert <cert>] [-tls_pkey <pkey>]
+                             [-tls_pass <password>] [-tls_verify] [-tls_cafile <cafile>]
+                             [-tls_capath <capath>] [-tls_ciphers <list>] [-tls | -notls]
+                             [-starttls | -nostarttls | -disallow_starttls | -nodisallow_starttls]
                              [-auto | -noauto] [-network <network>] [-host <hostname>]
-                             [-cmdspeed <ms>] [-cmdmax <count>] [-port <port>]
-                             <address> [<port> [<password>]] */
+                             [-cmdspeed <ms>] [-cmdmax <count>] [-port <port>] <address> [<port>
+                             [<password>]] */
 /* NOTE: -network replaces the old -ircnet flag. */
 static void sig_server_add_fill(IRC_SERVER_SETUP_REC *rec,
 				GHashTable *optlist)
@@ -85,6 +86,28 @@ static void sig_server_add_fill(IRC_SERVER_SETUP_REC *rec,
 	if (value != NULL && *value != '\0') rec->max_cmds_at_once = atoi(value);
 	value = g_hash_table_lookup(optlist, "querychans");
 	if (value != NULL && *value != '\0') rec->max_query_chans = atoi(value);
+	if (g_hash_table_lookup(optlist, "nodisallow_starttls") ||
+	    g_hash_table_lookup(optlist, "nostarttls"))
+		rec->starttls = STARTTLS_NOTSET;
+	if (g_hash_table_lookup(optlist, "disallow_starttls"))
+		rec->starttls = STARTTLS_DISALLOW;
+	if (g_hash_table_lookup(optlist, "starttls")) {
+		rec->starttls = STARTTLS_ENABLED;
+		rec->use_tls = 0;
+	}
+	if (g_hash_table_lookup(optlist, "nocap"))
+		rec->no_cap = 1;
+	if (g_hash_table_lookup(optlist, "cap"))
+		rec->no_cap = 0;
+}
+
+static void sig_server_waiting_info(IRC_SERVER_REC *server, const char *version)
+{
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	printformat(server, NULL, MSGLEVEL_CLIENTCRAP, IRCTXT_SERVER_WAITING_CAP_LS, server,
+	            version);
 }
 
 /* SYNTAX: SERVER LIST */
@@ -108,29 +131,35 @@ static void cmd_server_list(const char *data)
 			g_string_append(str, "autoconnect, ");
 		if (rec->no_proxy)
 			g_string_append(str, "noproxy, ");
-		if (rec->use_tls) {
+		if (rec->no_cap)
+			g_string_append(str, "nocap, ");
+		if (rec->starttls == STARTTLS_DISALLOW)
+			g_string_append(str, "disallow_starttls, ");
+		if (rec->starttls == STARTTLS_ENABLED)
+			g_string_append(str, "starttls, ");
+		if (rec->use_tls)
 			g_string_append(str, "tls, ");
-			if (rec->tls_cert) {
-				g_string_append_printf(str, "tls_cert: %s, ", rec->tls_cert);
-				if (rec->tls_pkey)
-					g_string_append_printf(str, "tls_pkey: %s, ", rec->tls_pkey);
-				if (rec->tls_pass)
-					g_string_append_printf(str, "(pass), ");
-			}
-			if (rec->tls_verify)
-				g_string_append(str, "tls_verify, ");
-			if (rec->tls_cafile)
-				g_string_append_printf(str, "tls_cafile: %s, ", rec->tls_cafile);
-			if (rec->tls_capath)
-				g_string_append_printf(str, "tls_capath: %s, ", rec->tls_capath);
-			if (rec->tls_ciphers)
-				g_string_append_printf(str, "tls_ciphers: %s, ", rec->tls_ciphers);
-			if (rec->tls_pinned_cert)
-				g_string_append_printf(str, "tls_pinned_cert: %s, ", rec->tls_pinned_cert);
-			if (rec->tls_pinned_pubkey)
-				g_string_append_printf(str, "tls_pinned_pubkey: %s, ", rec->tls_pinned_pubkey);
-
+		if (rec->tls_cert) {
+			g_string_append_printf(str, "tls_cert: %s, ", rec->tls_cert);
+			if (rec->tls_pkey)
+				g_string_append_printf(str, "tls_pkey: %s, ", rec->tls_pkey);
+			if (rec->tls_pass)
+				g_string_append_printf(str, "(pass), ");
 		}
+		if (!rec->tls_verify)
+			g_string_append(str, "notls_verify, ");
+		if (rec->tls_cafile)
+			g_string_append_printf(str, "tls_cafile: %s, ", rec->tls_cafile);
+		if (rec->tls_capath)
+			g_string_append_printf(str, "tls_capath: %s, ", rec->tls_capath);
+		if (rec->tls_ciphers)
+			g_string_append_printf(str, "tls_ciphers: %s, ", rec->tls_ciphers);
+		if (rec->tls_pinned_cert)
+			g_string_append_printf(str, "tls_pinned_cert: %s, ", rec->tls_pinned_cert);
+		if (rec->tls_pinned_pubkey)
+			g_string_append_printf(str, "tls_pinned_pubkey: %s, ",
+			                       rec->tls_pinned_pubkey);
+
 		if (rec->max_cmds_at_once > 0)
 			g_string_append_printf(str, "cmdmax: %d, ", rec->max_cmds_at_once);
 		if (rec->cmd_queue_speed > 0)
@@ -153,13 +182,20 @@ static void cmd_server_list(const char *data)
 void fe_irc_server_init(void)
 {
 	signal_add("server add fill", (SIGNAL_FUNC) sig_server_add_fill);
+	signal_add("server waiting cap ls", (SIGNAL_FUNC) sig_server_waiting_info);
 	command_bind("server list", NULL, (SIGNAL_FUNC) cmd_server_list);
 
-	command_set_options("server add", "-ircnet -network -cmdspeed -cmdmax -querychans");
+	command_set_options("server add",
+	                    "-ircnet -network -cmdspeed -cmdmax -querychans starttls "
+	                    "nostarttls disallow_starttls nodisallow_starttls cap nocap");
+	command_set_options("server modify",
+	                    "-ircnet -network -cmdspeed -cmdmax -querychans starttls nostarttls "
+	                    "disallow_starttls nodisallow_starttls cap nocap");
 }
 
 void fe_irc_server_deinit(void)
 {
 	signal_remove("server add fill", (SIGNAL_FUNC) sig_server_add_fill);
+	signal_remove("server waiting cap ls", (SIGNAL_FUNC) sig_server_waiting_info);
 	command_unbind("server list", (SIGNAL_FUNC) cmd_server_list);
 }
