@@ -19,18 +19,18 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "levels.h"
-#include "misc.h"
-#include "settings.h"
+#include <irssi/src/fe-common/irc/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
 
-#include "irc-servers.h"
-#include "modes.h"
-#include "ignore.h"
-#include "netsplit.h"
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/modes.h>
+#include <irssi/src/core/ignore.h>
+#include <irssi/src/irc/core/netsplit.h>
 
-#include "printtext.h"
+#include <irssi/src/fe-common/core/printtext.h>
 
 #define NETJOIN_WAIT_TIME 5 /* how many seconds to wait for the netsplitted JOIN messages to stop */
 #define NETJOIN_MAX_WAIT 30 /* how many seconds to wait for nick to join to the rest of the channels she was before the netsplit */
@@ -176,8 +176,7 @@ static void print_netjoins(NETJOIN_SERVER_REC *server, const char *filter_channe
 
 	/* save nicks to string, clear now_channels and remove the same
 	   channels from old_channels list */
-	channels = g_hash_table_new((GHashFunc) g_istr_hash,
-				    (GCompareFunc) g_istr_equal);
+	channels = g_hash_table_new((GHashFunc) i_istr_hash, (GCompareFunc) i_istr_equal);
 	for (tmp = server->netjoins; tmp != NULL; tmp = next) {
 		NETJOIN_REC *rec = tmp->data;
 
@@ -213,8 +212,7 @@ static void print_netjoins(NETJOIN_SERVER_REC *server, const char *filter_channe
 			}
 
 			/* remove the channel from old_channels too */
-			old = gslist_find_icase_string(rec->old_channels,
-						       realchannel);
+			old = i_slist_find_icase_string(rec->old_channels, realchannel);
 			if (old != NULL) {
 				void *data = old->data;
 				rec->old_channels =
@@ -341,7 +339,7 @@ static void msg_join(IRC_SERVER_REC *server, const char *channel,
 
 	/* if this was not a channel they split from, treat it normally */
 	if (netjoin != NULL) {
-		if (!gslist_find_icase_string(netjoin->old_channels, channel))
+		if (!i_slist_find_icase_string(netjoin->old_channels, channel))
 			return;
 	} else {
 		channels = split->channels;
