@@ -19,14 +19,15 @@
 */
 
 #include "module.h"
-#include "misc.h"
-#include "utf8.h"
-#include "formats.h"
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/utf8.h>
+#include <irssi/src/fe-common/core/formats.h>
 
-#include "gui-entry.h"
-#include "gui-printtext.h"
-#include "term.h"
-#include "recode.h"
+#include <irssi/src/fe-text/gui-entry.h>
+#include <irssi/src/fe-text/gui-printtext.h>
+#include <irssi/src/fe-text/term.h>
+#include <irssi/src/core/recode.h>
 
 #undef i_toupper
 #undef i_tolower
@@ -611,13 +612,13 @@ void gui_entry_insert_text(GUI_ENTRY_REC *entry, const char *str)
         entry_text_grow(entry, len);
 
         /* make space for the string */
-	g_memmove(entry->text + entry->pos + len, entry->text + entry->pos,
-		  (entry->text_len-entry->pos + 1) * sizeof(unichar));
+	memmove(entry->text + entry->pos + len, entry->text + entry->pos,
+	        (entry->text_len-entry->pos + 1) * sizeof(unichar));
 
 	/* make space for the color */
 	if (entry->uses_extents) {
-		g_memmove(entry->extents + entry->pos + len + 1, entry->extents + entry->pos + 1,
-			  (entry->text_len-entry->pos) * sizeof(char *));
+		memmove(entry->extents + entry->pos + len + 1, entry->extents + entry->pos + 1,
+		        (entry->text_len-entry->pos) * sizeof(char *));
 		for (i = 0; i < len; i++) {
 			entry->extents[entry->pos + i + 1] = NULL;
 		}
@@ -662,12 +663,12 @@ void gui_entry_insert_char(GUI_ENTRY_REC *entry, unichar chr)
 	entry_text_grow(entry, 1);
 
 	/* make space for the string */
-	g_memmove(entry->text + entry->pos + 1, entry->text + entry->pos,
-		  (entry->text_len-entry->pos + 1) * sizeof(unichar));
+	memmove(entry->text + entry->pos + 1, entry->text + entry->pos,
+	        (entry->text_len-entry->pos + 1) * sizeof(unichar));
 
 	if (entry->uses_extents) {
-		g_memmove(entry->extents + entry->pos + 1 + 1, entry->extents + entry->pos + 1,
-			  (entry->text_len-entry->pos) * sizeof(char *));
+		memmove(entry->extents + entry->pos + 1 + 1, entry->extents + entry->pos + 1,
+		        (entry->text_len-entry->pos) * sizeof(char *));
 		entry->extents[entry->pos + 1] = NULL;
 	}
 
@@ -772,11 +773,13 @@ static GUI_ENTRY_CUTBUFFER_REC *get_cutbuffer_rec(GUI_ENTRY_REC *entry, CUTBUFFE
 
 void gui_entry_erase(GUI_ENTRY_REC *entry, int size, CUTBUFFER_UPDATE_OP update_cutbuffer)
 {
+	gboolean clear_enabled;
 	size_t i, w = 0;
 
         g_return_if_fail(entry != NULL);
+	clear_enabled = settings_get_bool("empty_kill_clears_cutbuffer");
 
-	if (size == 0 || entry->pos < size)
+	if (entry->pos < size || (size == 0 && !clear_enabled))
 		return;
 
 	if (update_cutbuffer != CUTBUFFER_UPDATE_NOOP) {
@@ -792,54 +795,59 @@ void gui_entry_erase(GUI_ENTRY_REC *entry, int size, CUTBUFFER_UPDATE_OP update_
 		tmpcutbuffer = tmp->cutbuffer;
 		entry->append_next_kill = TRUE;
 		switch (update_cutbuffer) {
-			case CUTBUFFER_UPDATE_APPEND:
-				tmp->cutbuffer = g_new(unichar, cutbuffer_new_size+1);
-				memcpy(tmp->cutbuffer, tmpcutbuffer,
-				       tmp->cutbuffer_len * sizeof(unichar));
-				memcpy(tmp->cutbuffer + tmp->cutbuffer_len,
-				       entry->text + entry->pos - size, size * sizeof(unichar));
+		case CUTBUFFER_UPDATE_APPEND:
+			tmp->cutbuffer = g_new(unichar, cutbuffer_new_size + 1);
+			memcpy(tmp->cutbuffer, tmpcutbuffer, tmp->cutbuffer_len * sizeof(unichar));
+			memcpy(tmp->cutbuffer + tmp->cutbuffer_len, entry->text + entry->pos - size,
+			       size * sizeof(unichar));
 
-				tmp->cutbuffer_len = cutbuffer_new_size;
-				tmp->cutbuffer[cutbuffer_new_size] = '\0';
-				g_free(tmpcutbuffer);
-				break;
+			tmp->cutbuffer_len = cutbuffer_new_size;
+			tmp->cutbuffer[cutbuffer_new_size] = '\0';
+			g_free(tmpcutbuffer);
+			break;
 
-			case CUTBUFFER_UPDATE_PREPEND:
-				tmp->cutbuffer = g_new(unichar, cutbuffer_new_size+1);
-				memcpy(tmp->cutbuffer, entry->text + entry->pos - size,
-				       size * sizeof(unichar));
-				memcpy(tmp->cutbuffer + size, tmpcutbuffer,
-				       tmp->cutbuffer_len * sizeof(unichar));
+		case CUTBUFFER_UPDATE_PREPEND:
+			tmp->cutbuffer = g_new(unichar, cutbuffer_new_size + 1);
+			memcpy(tmp->cutbuffer, entry->text + entry->pos - size,
+			       size * sizeof(unichar));
+			memcpy(tmp->cutbuffer + size, tmpcutbuffer,
+			       tmp->cutbuffer_len * sizeof(unichar));
 
-				tmp->cutbuffer_len = cutbuffer_new_size;
-				tmp->cutbuffer[cutbuffer_new_size] = '\0';
-				g_free(tmpcutbuffer);
-				break;
+			tmp->cutbuffer_len = cutbuffer_new_size;
+			tmp->cutbuffer[cutbuffer_new_size] = '\0';
+			g_free(tmpcutbuffer);
+			break;
 
-			case CUTBUFFER_UPDATE_REPLACE:
-				/* put erased text to cutbuffer */
-				if (tmp->cutbuffer_len < size) {
-					g_free(tmp->cutbuffer);
-					tmp->cutbuffer = g_new(unichar, size+1);
-				}
+		case CUTBUFFER_UPDATE_REPLACE:
+			/* put erased text to cutbuffer */
+			if (tmp->cutbuffer_len < size || tmp->cutbuffer == NULL) {
+				g_free(tmp->cutbuffer);
+				tmp->cutbuffer = g_new(unichar, size + 1);
+			}
 
-				tmp->cutbuffer_len = size;
-				tmp->cutbuffer[size] = '\0';
-				memcpy(tmp->cutbuffer, entry->text + entry->pos - size, size * sizeof(unichar));
-				break;
+			tmp->cutbuffer_len = size;
+			tmp->cutbuffer[size] = '\0';
+			memcpy(tmp->cutbuffer, entry->text + entry->pos - size,
+			       size * sizeof(unichar));
+			break;
 
-			case CUTBUFFER_UPDATE_NOOP:
-				/* cannot happen, handled in "if" */
-				break;
+		case CUTBUFFER_UPDATE_NOOP:
+			/* cannot happen, handled in "if" */
+			break;
 		}
 	}
 
+	if (size == 0) {
+		/* we just wanted to clear the cutbuffer */
+		return;
+	}
+
 	if (entry->utf8)
 		while (entry->pos > size + w && i_wcwidth(entry->text[entry->pos - size - w]) == 0)
 			w++;
 
-	g_memmove(entry->text + entry->pos - size, entry->text + entry->pos,
-		  (entry->text_len-entry->pos+1) * sizeof(unichar));
+	memmove(entry->text + entry->pos - size, entry->text + entry->pos,
+	        (entry->text_len-entry->pos+1) * sizeof(unichar));
 
 	if (entry->uses_extents) {
 		for (i = entry->pos - size; i < entry->pos; i++) {
@@ -847,8 +855,8 @@ void gui_entry_erase(GUI_ENTRY_REC *entry, int size, CUTBUFFER_UPDATE_OP update_
 				g_free(entry->extents[i+1]);
 			}
 		}
-		g_memmove(entry->extents + entry->pos - size + 1, entry->extents + entry->pos + 1,
-			  (entry->text_len - entry->pos) * sizeof(void *)); /* no null terminator here */
+		memmove(entry->extents + entry->pos - size + 1, entry->extents + entry->pos + 1,
+		        (entry->text_len - entry->pos) * sizeof(void *)); /* no null terminator here */
 		for (i = 0; i < size; i++) {
 			entry->extents[entry->text_len - i] = NULL;
 		}
@@ -876,16 +884,16 @@ void gui_entry_erase_cell(GUI_ENTRY_REC *entry)
 		while (entry->pos+size < entry->text_len &&
 		       i_wcwidth(entry->text[entry->pos+size]) == 0) size++;
 
-	g_memmove(entry->text + entry->pos, entry->text + entry->pos + size,
-	          (entry->text_len-entry->pos-size+1) * sizeof(unichar));
+	memmove(entry->text + entry->pos, entry->text + entry->pos + size,
+	        (entry->text_len-entry->pos-size+1) * sizeof(unichar));
 
 	if (entry->uses_extents) {
 		int i;
 		for (i = 0; i < size; i++) {
 			g_free(entry->extents[entry->pos + i + 1]);
 		}
-		g_memmove(entry->extents + entry->pos + 1, entry->extents + entry->pos + size + 1,
-			  (entry->text_len-entry->pos-size) * sizeof(char *));
+		memmove(entry->extents + entry->pos + 1, entry->extents + entry->pos + size + 1,
+		        (entry->text_len-entry->pos-size) * sizeof(char *));
 		for (i = 0; i < size; i++) {
 			entry->extents[entry->text_len - i] = NULL;
 		}
@@ -1498,6 +1506,7 @@ void gui_entry_set_text_and_extents(GUI_ENTRY_REC *entry, GSList *list)
 
 void gui_entry_init(void)
 {
+	settings_add_bool("lookandfeel", "empty_kill_clears_cutbuffer", FALSE);
 }
 
 void gui_entry_deinit(void)
