@@ -19,19 +19,19 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "commands.h"
-#include "recode.h"
-#include "network.h"
-#include "net-sendbuffer.h"
-#include "misc.h"
-#include "settings.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/recode.h>
+#include <irssi/src/core/network.h>
+#include <irssi/src/core/net-sendbuffer.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
 
-#include "irc-servers.h"
-#include "irc-queries.h"
-#include "masks.h"
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/irc-queries.h>
+#include <irssi/src/core/masks.h>
 
-#include "dcc-chat.h"
+#include <irssi/src/irc/dcc/dcc-chat.h>
 
 static char *dcc_chat_get_new_id(const char *nick)
 {
@@ -318,13 +318,17 @@ void dcc_chat_input(CHAT_DCC_REC *dcc)
 		}
 
 		if (ret > 0) {
+			SERVER_REC *server;
 			char *recoded;
 
 			dcc->transfd += ret;
 
-			recoded = recode_in(SERVER(dcc->server), str, dcc->nick);
+			server = SERVER(dcc->server);
+			recoded = recode_in(server, str, dcc->nick);
 			signal_emit("dcc chat message", 2, dcc, recoded);
 			g_free(recoded);
+			if (server != NULL)
+				server_meta_clear_all(server);
 		}
 	} while (ret > 0);
 }
@@ -355,13 +359,12 @@ static void dcc_chat_listen(CHAT_DCC_REC *dcc)
 	memcpy(&dcc->addr, &ip, sizeof(IPADDR));
 	net_ip2host(&dcc->addr, dcc->addrstr);
 	dcc->port = port;
-	dcc->tagread = g_input_add(handle, G_INPUT_READ,
-				   (GInputFunction) dcc_chat_input, dcc);
+	dcc->tagread = i_input_add(handle, I_INPUT_READ, (GInputFunction) dcc_chat_input, dcc);
 
 	signal_emit("dcc connected", 1, dcc);
 }
 
-/* callback: DCC CHAT - net_connect_nonblock() finished */
+/* callback: DCC CHAT - connect finished */
 static void sig_chat_connected(CHAT_DCC_REC *dcc)
 {
 	g_return_if_fail(IS_DCC_CHAT(dcc));
@@ -379,8 +382,7 @@ static void sig_chat_connected(CHAT_DCC_REC *dcc)
 
 	dcc->starttime = time(NULL);
 	dcc->sendbuf = net_sendbuffer_create(dcc->handle, 0);
-	dcc->tagread = g_input_add(dcc->handle, G_INPUT_READ,
-				   (GInputFunction) dcc_chat_input, dcc);
+	dcc->tagread = i_input_add(dcc->handle, I_INPUT_READ, (GInputFunction) dcc_chat_input, dcc);
 
 	signal_emit("dcc connected", 1, dcc);
 }
@@ -397,9 +399,8 @@ static void dcc_chat_connect(CHAT_DCC_REC *dcc)
 
 	dcc->handle = dcc_connect_ip(&dcc->addr, dcc->port);
 	if (dcc->handle != NULL) {
-		dcc->tagconn = g_input_add(dcc->handle,
-					   G_INPUT_WRITE | G_INPUT_READ,
-					   (GInputFunction) sig_chat_connected, dcc);
+		dcc->tagconn = i_input_add(dcc->handle, I_INPUT_WRITE | I_INPUT_READ,
+		                           (GInputFunction) sig_chat_connected, dcc);
 	} else {
 		/* error connecting */
 		signal_emit("dcc error connect", 1, dcc);
@@ -428,8 +429,8 @@ static void dcc_chat_passive(CHAT_DCC_REC *dcc)
 		cmd_return_error(CMDERR_ERRNO);
 
 	dcc->handle = handle;
-	dcc->tagconn = g_input_add(dcc->handle, G_INPUT_READ,
-				   (GInputFunction) dcc_chat_listen, dcc);
+	dcc->tagconn =
+	    i_input_add(dcc->handle, I_INPUT_READ, (GInputFunction) dcc_chat_listen, dcc);
 
 	/* Let's send the reply to the other client! */
 	dcc_ip2str(&own_ip, host);
@@ -508,8 +509,7 @@ static void cmd_dcc_chat(const char *data, IRC_SERVER_REC *server)
 
 		dcc->handle = handle;
 		dcc->tagconn =
-			g_input_add(dcc->handle, G_INPUT_READ,
-				    (GInputFunction) dcc_chat_listen, dcc);
+		    i_input_add(dcc->handle, I_INPUT_READ, (GInputFunction) dcc_chat_listen, dcc);
 
 		/* send the chat request */
 		signal_emit("dcc request send", 1, dcc);
